@@ -1,5 +1,6 @@
 import os
 import json
+import logging
 
 from enum import Enum
 from functools import wraps
@@ -8,13 +9,14 @@ from typing import List, Union
 
 from pandas import Series
 from numpy import ndarray
-from urllib3 import Retry
 
 from tune_the_model.resource import (
     TuneTheModelAPI,
     TuneTheModelException,
 )
 
+log = logging.getLogger(__name__)
+
 
 MINIMUM_ENTRIES = 32
 
@@ -60,12 +62,14 @@ class TuneTheModelFile():
     _status: str = None
     _task_type: str = None
     _file_name: str = None
+    _upload_url: str = None
 
     def __init__(self, file_id: str, status: str, task_type: str, file_name: str, *args, **kwargs):
         self._id = file_id
         self._status = status
         self._task_type = task_type
         self._file_name = file_name
+        self._upload_url = kwargs.get("upload_url")
 
     @classmethod
     def from_dict(cls, model: dict) -> 'TuneTheModelFile':
@@ -146,11 +150,6 @@ class TuneTheModelFile():
         r = TuneTheModelAPI.delete_file(self._id)
         return r
 
-    @inited
-    def wait_for_uploading_finish(self, sleep_for: int = 1):
-        while self.status is not TuneTheModelFileStatus.READY:
-            sleep(sleep_for)
-
     @inited
     def upload(
         self,
@@ -183,16 +182,7 @@ class TuneTheModelFile():
 
         data = json.dumps(data, default=_default)
 
-        def MB(i):
-            return i / 1024 ** 2
-
-        upper_limit = 8 * 1024 ** 2
-        if len(data) > upper_limit:
-            raise TuneTheModelException(
-                f"Payload exceeds the limit {MB(upper_limit):0.1f}MB with size of {MB(len(data)):0.2f}MB"
-            )
-
-        return TuneTheModelAPI.upload_file(self._id, data=data)
+        return TuneTheModelAPI.upload_file(self._upload_url, data=data)
 
     @classmethod
     def files(cls) -> List['TuneTheModel']:
@@ -366,9 +356,6 @@ class TuneTheModel():
         validate_file = TuneTheModelFile.create("val", self.type)
         validate_file.upload(validate_X, validate_y)
 
-        train_file.wait_for_uploading_finish()
-        validate_file.wait_for_uploading_finish()
-
         self.bind(train_file, validate_file)
 
         if self.status is not TuneTheModelStatus.DATASETS_LOADED:
@@ -392,7 +379,7 @@ class TuneTheModel():
         Raises:
             TuneTheModelException: If anything bad happens.
         """
-        r = TuneTheModelAPI.generate(self._id, {"input" : input})
+        r = TuneTheModelAPI.generate(self._id, {"input": input})
         return r["answer"]["responses"][0]["response"]
 
     @inited
@@ -413,7 +400,11 @@ class TuneTheModel():
 
     @inited
     def wait_for_training_finish(self, sleep_for: int = 60):
+        log.info("Waiting until model is ready")
         while self.status is not TuneTheModelStatus.READY:
+            if (self.status is TuneTheModelStatus.FAILED):
+                raise TuneTheModelException(
+                    "Something went wrong during the fit process. Please, contact us")
             sleep(sleep_for)
 
     @inited
@@ -598,7 +589,7 @@ def tune_classifier(
     return model
 
 
-def generate(input: str, model_id: str="default-generator"):
+def generate(input: str, model_id: str = "default-generator"):
     """Generates a suffix based on an input prefix.
 
     Args:
